@@ -69,9 +69,11 @@
 
 
 /* bug workaround (double click signals click first)*/
-static gboolean skip = FALSE;
+static gboolean skip_second_release = FALSE;
 static GtkTreeRowReference *title_reference=NULL;
 static mousedown=FALSE;
+static GtkTreePath *clickpath;
+static gboolean unsel(GtkTreeModel *,GtkTreePath *, GtkTreeIter *, gpointer);
 
 extern int stop;
 /*XXX for gtk bug workaround with flashing treeview on expand/collapse
@@ -101,6 +103,9 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 #ifdef DEBUG
 	printf("DBG: open_dir ...\n");
 #endif
+    if (clickpath){
+        clear_path_from_selection_list(treeview,clickpath);
+    }
     mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
 
     if (!valid_iter(treeview,iter)){
@@ -418,6 +423,9 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
     tree_details_t *tree_details = get_tree_details(treeview);
 
     mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
+    if (clickpath){
+        clear_path_from_selection_list(treeview,clickpath);
+    }
 
     if (!valid_iter(treeview,iter)){
 	    local_monitor(treeview,TRUE);
@@ -519,6 +527,7 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
 
 gboolean on_mouse_move(GtkWidget *widget,GdkEventMotion *event, gpointer user_data)
 {
+#if 0
 	static gboolean reselected=FALSE;
 	if (!mousedown) {
 		reselected=FALSE;
@@ -527,9 +536,21 @@ gboolean on_mouse_move(GtkWidget *widget,GdkEventMotion *event, gpointer user_da
 		reselected=TRUE;
  		reselect_dnd_list((GtkTreeView *)widget);
 	}
+#endif
 	return FALSE;
 }
 	
+
+static int reselecttimer=0;
+static gint do_reselect(gpointer data){
+	GtkTreeView * treeview=(GtkTreeView * )data;
+ 	reselect_dnd_list(treeview);
+		    /*printf("DBG:at reselecttimer = %d\n",reselecttimer );*/
+	reselecttimer=0;
+	return FALSE;
+}
+
+	
 gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
@@ -542,9 +563,9 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
    mousedown=FALSE; 
     if(tree_details->loading)	return FALSE;
 
-   if(event->button == 1 && skip) 
+   if(event->button == 1 && skip_second_release) 
     {
-	skip = FALSE;
+	skip_second_release = FALSE;
 	return FALSE;
     }
     switch (event->button)
@@ -564,7 +585,7 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
 		gtk_tree_model_get_iter(treemodel, &iter, treepath);
 
 		gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
-		readable_path=g_strdup(FILENAME(en));
+		readable_path=g_path_get_basename(en->path);
 		if (IS_NETWORK_TYPE(en->type)&&!IS_SAMBA_SERVER(en->subtype))
 			ascii_readable(readable_path);
 		print_status_tmp(treeview, resolve_icon_small(en), 
@@ -639,10 +660,9 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
     {
 	cancel_input(treeview);
     }
-
+    skip_second_release = FALSE;
 
     if (double_treeview ){
-      static gboolean unsel(GtkTreeModel *,GtkTreePath *, GtkTreeIter *, gpointer);
       if (treeview == tree_details->treeview){
         /* unselect all in auxiliary treeview */
 	GtkTreeView *treeview2=(GtkTreeView *)lookup_widget(tree_details->window,"treeview2");
@@ -655,14 +675,19 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
       }
     }
     
-    /*//if(event->state==GDK_CONTROL_MASK &&(event->button == 4 || event->button == 5)){
+    /*//if(event->state&GDK_CONTROL_MASK &&(event->button == 4 || event->button == 5)){
     if((event->button == 4 || event->button == 5)){
       printf("DBG:ctr button %d, state=%d\n",event->button,event->state);
 	return TRUE;
     }*/
     
-    if((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
+    if((event->type == GDK_2BUTTON_PRESS) && (event->button == 1) && !(event->state&GDK_CONTROL_MASK))
     {				/* this works because item is selected at first click of double click */
+	skip_second_release=TRUE; /* skip the second button release event */
+	if (reselecttimer)g_source_remove(reselecttimer);
+		    /*printf("DBG:canceled reselecttimer = %d\n",reselecttimer );*/
+	reselecttimer=0;
+
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
 	if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
 	{
@@ -671,7 +696,6 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 	    type = en->type;
 	    if (IS_APP_TYPE(en->type) && !IS_ROOT_TYPE(en->type)){
 		double_click_run(tree_details,en);
-    		skip = TRUE;
 		goto done;
 	    }
 	    if (IS_FSTAB_TYPE(en->type) && IS_XF_FSTAB(en->type)){
@@ -710,11 +734,9 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
     				SET_IN_TERM(en->subtype);
 				double_click_run(tree_details,en);
 			}
-    			skip = TRUE;
 		} else 
 		{      /* open with */
 			double_click_open_with(tree_details,en);
-    			skip = TRUE;
 		}
 	    }
        	    else if(gtk_tree_model_iter_children(treemodel, &child, &iter)){
@@ -737,6 +759,9 @@ done:
 	    break;
 	case 3:
 	    clear_dnd_selection_list(); /* only currently selected items */
+	    if (!(event->state&GDK_CONTROL_MASK)){
+    		gtk_tree_model_foreach(treemodel,unsel,selection);
+	    }
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
 				    event->y, &treepath, 
 				    NULL, NULL, NULL))
@@ -762,18 +787,28 @@ done:
 	    /* this enables dnd(right), but also selection(wrong)*/
 #endif
 	case 1:
-	  if (event->state!=GDK_CONTROL_MASK && event->state!=GDK_SHIFT_MASK) {
+	  if (!(event->state&GDK_CONTROL_MASK) && !(event->state&GDK_SHIFT_MASK)) {
 	    clear_dnd_selection_list();
     	    gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
-	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
-				    event->y, &treepath, 
-				    NULL, NULL, NULL))	
+	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &treepath, NULL, NULL, NULL))	
 	    if (!is_in_dnd_selection(treepath)){
 		if (gtk_tree_model_get_iter (treemodel,&iter,treepath)) 
 		{
 		  get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);
 		}
+		if (clickpath) gtk_tree_path_free(clickpath);
+		clickpath=gtk_tree_path_copy(treepath);
 		gtk_tree_path_free(treepath);
+	    } else {
+		if (clickpath) gtk_tree_path_free(clickpath);
+		clickpath=NULL;
+	    }
+	    if (!(tree_details->preferences & ENABLE_GTK_CLICK)) 
+	    {
+		    reselecttimer = g_timeout_add_full(0, 150, (GtkFunction) do_reselect, (gpointer) treeview, NULL);
+#ifdef DEBUG
+		    printf("DBG:reselecttimer = %d\n",reselecttimer );
+#endif
 	    }
 	  }
 	    return FALSE;
